#!/bin/bash

# SET BUILD OPTIONS
ASM_OPTIONS=""
case ${ARCH} in
x86 | x86-64)
  ASM_OPTIONS="--enable-hardware-optimizations --enable-intel-sse=yes"
  ;;
arm-v7a-neon | arm64-v8a)
  ASM_OPTIONS="--enable-hardware-optimizations --enable-arm-neon=yes"
  ;;
arm-v7a)
  # hardware-optimizations not enabled because
  # when --enable-hardware-optimizations is added
  # make tries to build arm-neon specific instructions, which breaks compilation
  ASM_OPTIONS="--enable-arm-neon=no"
  ;;
esac

# ALWAYS CLEAN THE PREVIOUS BUILD
make distclean 2>/dev/null 1>/dev/null

# REGENERATE BUILD FILES IF NECESSARY OR REQUESTED
if [[ ! -f "${BASEDIR}"/src/"${LIB_NAME}"/configure ]] || [[ ${RECONF_libpng} -eq 1 ]]; then
  autoreconf_library "${LIB_NAME}"
fi

./configure \
  --prefix="${LIB_INSTALL_PREFIX}" \
  --with-pic \
  --with-sysroot="${ANDROID_SYSROOT}" \
  --enable-static \
  --disable-shared \
  --disable-fast-install \
  --disable-unversioned-libpng-pc \
  --disable-unversioned-libpng-config \
  ${ASM_OPTIONS} \
  --host="${HOST}" || return 1

make -j$(get_cpu_count) || return 1

make install || return 1

# MANUALLY COPY PKG-CONFIG FILES
cp ./*.pc "${INSTALL_PKG_CONFIG_DIR}" || return 1
